/**
  **************************************************************************
  * @file     main.c
  * @version  v2.0.2
  * @date     2021-10-27
  * @brief    main program
  **************************************************************************
  *                       Copyright notice & Disclaimer
  *
  * The software Board Support Package (BSP) that is made available to 
  * download from Artery official website is the copyrighted work of Artery. 
  * Artery authorizes customers to use, copy, and distribute the BSP 
  * software and its related documentation for the purpose of design and 
  * development in conjunction with Artery microcontrollers. Use of the 
  * software is governed by this copyright notice and the following disclaimer.
  *
  * THIS SOFTWARE IS PROVIDED ON "AS IS" BASIS WITHOUT WARRANTIES,
  * GUARANTEES OR REPRESENTATIONS OF ANY KIND. ARTERY EXPRESSLY DISCLAIMS,
  * TO THE FULLEST EXTENT PERMITTED BY LAW, ALL EXPRESS, IMPLIED OR
  * STATUTORY OR OTHER WARRANTIES, GUARANTEES OR REPRESENTATIONS,
  * INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT.
  *
  **************************************************************************
  */

#include "at32f403a_407_clock.h"
#include "stdio.h"

/** @addtogroup UTILITIES_examples
  * @{
  */
	
/** @addtogroup 403A_TMR_pwm_input_test TMR_pwm_input
  * @{
  */

tmr_input_config_type tmr_ic_init_structure;
crm_clocks_freq_type crm_clocks_freq_struct = {0};

__IO uint32_t Counter_1 = 0;
__IO uint32_t Counter_2 = 0;
__IO uint32_t Last_Counter_1 = 0;
__IO uint32_t Last_Counter_2 = 0;
uint32_t SystemCoreClock = 0;

void crm_configuration(void);
void gpio_configuration(void);
void nvic_configuration(void);
void uart_init(uint32_t baudrate);

/**
  * @brief  configure the tmr pins.
  * @param  none
  * @retval none
  */
void gpio_configuration(void)
{
  gpio_init_type gpio_init_struct;

  gpio_default_para_init(&gpio_init_struct);

  gpio_init_struct.gpio_pins = GPIO_PINS_0;
  gpio_init_struct.gpio_mode = GPIO_MODE_INPUT;
  gpio_init_struct.gpio_drive_strength = GPIO_DRIVE_STRENGTH_STRONGER;
  gpio_init(GPIOA, &gpio_init_struct);
}

/**
  * @brief  configures the different system clocks.
  * @param  none
  * @retval none
  */
void crm_configuration(void)
{
  /* tmr clock enable */
  crm_periph_clock_enable(CRM_TMR1_PERIPH_CLOCK, TRUE);
  crm_periph_clock_enable(CRM_TMR2_PERIPH_CLOCK, TRUE);
	crm_periph_clock_enable(CRM_TMR5_PERIPH_CLOCK, TRUE);
	
  /* gpioa clock enable */
  crm_periph_clock_enable(CRM_GPIOA_PERIPH_CLOCK, TRUE);
}

/**
  * @brief  configures the nested vectored interrupt controller.
  * @param  none
  * @retval none
  */
void nvic_configuration(void)
{
  nvic_priority_group_config(NVIC_PRIORITY_GROUP_2);
	
	#if defined(high_frequency_test)
  nvic_irq_enable(TMR5_GLOBAL_IRQn, 0, 0);

  #elif defined(low_frequency_test)	
  nvic_irq_enable(TMR2_GLOBAL_IRQn, 1, 1);
	
  #elif defined(duty_ration_test)
  nvic_irq_enable(TMR1_OVF_TMR10_IRQn, 0, 0);
	
  #endif
}

/**
  * @brief  initialize print usart
  * @param  baudrate: uart baudrate
  * @retval none
  */
void uart_init(uint32_t baudrate)
{
  gpio_init_type gpio_init_struct;

  /* enable the uart1 and gpio clock */
  crm_periph_clock_enable(CRM_USART1_PERIPH_CLOCK, TRUE);
  crm_periph_clock_enable(CRM_GPIOA_PERIPH_CLOCK, TRUE);

  gpio_default_para_init(&gpio_init_struct);

  /* configure the uart1 tx pin */
  gpio_init_struct.gpio_drive_strength = GPIO_DRIVE_STRENGTH_STRONGER;
  gpio_init_struct.gpio_out_type = GPIO_OUTPUT_PUSH_PULL;
  gpio_init_struct.gpio_mode = GPIO_MODE_MUX;
  gpio_init_struct.gpio_pins = GPIO_PINS_9;
  gpio_init_struct.gpio_pull = GPIO_PULL_NONE;
  gpio_init(GPIOA, &gpio_init_struct);

  /* configure uart param */
  usart_init(USART1, baudrate, USART_DATA_8BITS, USART_STOP_1_BIT);
  usart_transmitter_enable(USART1, TRUE);
  usart_enable(USART1, TRUE);
}

/* suport printf function, usemicrolib is unnecessary */
#ifdef __CC_ARM
  #pragma import(__use_no_semihosting)
  struct __FILE
  {
    int handle;
  };

  FILE __stdout;

  void _sys_exit(int x)
  {
    x = x;
  }
#endif

#ifdef __GNUC__
  /* with gcc/raisonance, small printf (option ld linker->libraries->small printf set to 'yes') calls __io_putchar() */
  #define PUTCHAR_PROTOTYPE int __io_putchar(int ch)
#else
  #define PUTCHAR_PROTOTYPE int fputc(int ch, FILE *f)
#endif /* __gnuc__ */

/**
  * @brief  retargets the c library printf function to the usart.
  * @param  none
  * @retval none
  */
PUTCHAR_PROTOTYPE
{
  while(usart_flag_get(USART1, USART_TDBE_FLAG) == RESET);
  usart_data_transmit(USART1, ch);
  return ch;
}

/**
  * @brief  main function.
  * @param  none
  * @retval none
  */
int main(void)
{
  /* system clocks configuration */
  system_clock_config();
  crm_configuration();

	/* get system clock */
	#if defined(low_frequency_test)
  crm_clocks_freq_get(&crm_clocks_freq_struct);
	SystemCoreClock = crm_clocks_freq_struct.sclk_freq;
  #endif	
	
  /* nvic configuration */
	nvic_configuration();

  /* gpio configuration */
  gpio_configuration();

	/* uart configuration */
  uart_init(115200);

	#if defined(high_frequency_test)
	
  /* Init TMR2, External signal as clock*/
	tmr_32_bit_function_enable(TMR2, TRUE);	
	tmr_base_init(TMR2, 0xFFFFFFFF, 0);
  tmr_cnt_dir_set(TMR2, TMR_COUNT_UP);

  /* select the tmr2 input trigger: C1DF1 */
	tmr_trigger_input_select(TMR2, TMR_SUB_INPUT_SEL_C1DF1);
	/* select the sub mode: external clock mode a */
	tmr_sub_mode_select(TMR2, TMR_SUB_EXTERNAL_CLOCK_MODE_A);
	
  /* Init TMR5, Create 1s per interrupt*/	
	tmr_base_init(TMR5, 24000-1, 10000-1);		//240MHz
  tmr_cnt_dir_set(TMR5, TMR_COUNT_UP);
	tmr_flag_clear(TMR5, TMR_OVF_FLAG);

  /* TMR enable counter */
	tmr_counter_enable(TMR2, TRUE);
	tmr_counter_enable(TMR5, TRUE);

  /* Enable the TMR5 Update Interrupt */
	tmr_interrupt_enable(TMR5, TMR_OVF_INT, TRUE);
	
  #elif defined(low_frequency_test)
	
  /* Init TMR2 . Enable Input Caputer*/
	tmr_32_bit_function_enable(TMR2, TRUE);		
	tmr_base_init(TMR2, 0xFFFFFFFF, 0);
  tmr_cnt_dir_set(TMR2, TMR_COUNT_UP);

	/* Init tmr input channel */
	tmr_input_default_para_init(&tmr_ic_init_structure);
  tmr_input_channel_init(TMR2, &tmr_ic_init_structure, TMR_CHANNEL_INPUT_DIV_1);

  /* Enable the TMR2 CC1 Interrupt */
	tmr_interrupt_enable(TMR2, TMR_C1_INT, TRUE);
	
  /* TMR enable counter */
	tmr_counter_enable(TMR2, TRUE);

#elif defined(duty_ration_test)
	
  /* Init TMR1, Create 1s per interrupt*/
	tmr_base_init(TMR1, 24000-1, 10000-1);
  tmr_cnt_dir_set(TMR1, TMR_COUNT_UP);	
	tmr_repetition_counter_set(TMR1, 0);
	
	tmr_flag_clear(TMR1, TMR_OVF_FLAG);	
	tmr_interrupt_enable(TMR1, TMR_OVF_INT, TRUE);	

  /* Init TMR2, External signal as Gate signal*/
	tmr_32_bit_function_enable(TMR2, TRUE);		
	tmr_base_init(TMR2, 0xFFFFFFFF, 0);
  tmr_cnt_dir_set(TMR2, TMR_COUNT_UP);
	
	tmr_input_default_para_init(&tmr_ic_init_structure);
	tmr_input_channel_init(TMR2, &tmr_ic_init_structure, TMR_CHANNEL_INPUT_DIV_1);
  /* select the tmr2 input trigger: C1DF1 */
	tmr_trigger_input_select(TMR2, TMR_SUB_INPUT_SEL_C1DF1);
	/* select the sub mode: hang mode */
	tmr_sub_mode_select(TMR2, TMR_SUB_HANG_MODE);

  /* Init TMR5, External signal as Gate signal*/
	tmr_32_bit_function_enable(TMR5, TRUE);		
	tmr_base_init(TMR5, 0xFFFFFFFF, 0);
  tmr_cnt_dir_set(TMR5, TMR_COUNT_UP);
	
	tmr_input_default_para_init(&tmr_ic_init_structure);
  tmr_ic_init_structure.input_polarity_select = TMR_INPUT_FALLING_EDGE;	
	tmr_input_channel_init(TMR5, &tmr_ic_init_structure, TMR_CHANNEL_INPUT_DIV_1);
  /* select the tmr5 input trigger: C1DF1 */
	tmr_trigger_input_select(TMR5, TMR_SUB_INPUT_SEL_C1DF1);
	/* select the sub mode: hang mode */
	tmr_sub_mode_select(TMR5, TMR_SUB_HANG_MODE);	
	
  /* TMR enable counter */
	tmr_counter_enable(TMR2, TRUE);
	tmr_counter_enable(TMR5, TRUE);	
	tmr_counter_enable(TMR1, TRUE);	
	
  #endif	
	
	printf("Make sure connected with external signal\r\n");
	
  while(1)
  {
  }
}

/**
  * @}
  */ 

/**
  * @}
  */ 
